pub struct BidiStream<B>where
B: Buf,{
pub(crate) send: SendStream<B>,
pub(crate) recv: RecvStream,
}Expand description
Quinn-backed bidirectional stream
Implements [quic::BidiStream] which allows the stream to be split into two structs each implementing one direction.
Fields
send: SendStream<B>recv: RecvStreamTrait Implementations
Sourceimpl<B> BidiStream<B> for BidiStream<B>where B: Buf,
impl<B> BidiStream<B> for BidiStream<B>where B: Buf,
Sourcetype SendStream = SendStream<B>
type SendStream = SendStream<B>
The type for the send half.
Sourcetype RecvStream = RecvStream
type RecvStream = RecvStream
The type for the receive half.
Sourceimpl<B: Buf> RecvStream for BidiStream<B>
impl<B: Buf> RecvStream for BidiStream<B>
Sourceimpl<B> SendStream<B> for BidiStream<B>where B: Buf,
impl<B> SendStream<B> for BidiStream<B>where B: Buf,
Sourcefn poll_ready( &mut self, cx: &mut Context<'_>, ) -> Poll<Result<(), StreamErrorIncoming>>
fn poll_ready( &mut self, cx: &mut Context<'_>, ) -> Poll<Result<(), StreamErrorIncoming>>
Polls if the stream can send more data.
Sourcefn poll_finish( &mut self, cx: &mut Context<'_>, ) -> Poll<Result<(), StreamErrorIncoming>>
fn poll_finish( &mut self, cx: &mut Context<'_>, ) -> Poll<Result<(), StreamErrorIncoming>>
Poll to finish the sending side of the stream.